@@ -283,6 +283,9 @@ __END_TEMPLATE__
 
 #define FUT_NEW_LWP_CODE                        0xfff6
 
+#define FUT_START_FLUSH_CODE                    0xfff5
+#define FUT_STOP_FLUSH_CODE                     0xfff4
+
 #define FUT_RQS_NEWLEVEL			0xffef
 #define FUT_RQS_NEWLWPRQ			0xffee
 #define FUT_RQS_NEWRQ				0xffed
@@ -307,7 +310,7 @@ extern int fut_done(void );
 extern int fut_keychange( int how, unsigned int keymask,
 			unsigned int threadid );
 extern int fut_reset( unsigned int keymask, unsigned int threadid );
-extern int fut_getbuffer( int *nints, unsigned long **buffer );
+extern int fut_getbuffer( int *nints, unsigned long **buffer, unsigned long *next_slot);
 
 extern int fut_header( unsigned long head, ... );
 
