@@ -7,11 +7,11 @@
 #  ifdef USE_GETTID
 #    include <unistd.h>
 #    include <sys/syscall.h>	/* For SYS_xxx definitions */
-#  else
-#    include <pthread.h>
 #  endif
 #endif
 
+#include <pthread.h>
+
 #ifdef HAVE_CLOCK_GETTIME
 #include <time.h>
 #else
@@ -181,27 +181,12 @@ fut_header (unsigned long code, ... )
 #endif
 
 static unsigned long trash_buffer[8];
+extern int allow_fut_flush;
+extern pthread_mutex_t fut_flush_lock;
 
-unsigned long* fut_getstampedbuffer(unsigned long code, 
-				    int size)
+unsigned long* __fut_record_event(fxt_trace_user_raw_t * rec,
+					 unsigned long code)
 {
-	unsigned long *prev_slot, *next_slot;
-
-	do {
-		prev_slot=(unsigned long *)fut_next_slot;
-		next_slot=(unsigned long*)((unsigned long)prev_slot+size);
-	} while (prev_slot != ma_cmpxchg(&fut_next_slot, 
-					 prev_slot, next_slot));
-
-	if (tbx_unlikely(next_slot > fut_last_slot)) {
-		fut_active=0;
-		/* Pourquoi on restaure ici ? Pas de race condition possible ? */
-		fut_next_slot=prev_slot;
-		return trash_buffer;
-	}
-
-	fxt_trace_user_raw_t *rec=(fxt_trace_user_raw_t *)prev_slot;
-
 	unsigned long long tick;
 #ifdef HAVE_CLOCK_GETTIME
 	struct timespec tp;
@@ -234,4 +219,50 @@ unsigned long* fut_getstampedbuffer(unsigned long code,
 	return (unsigned long*)(rec->args);
 }
 
+unsigned long* fut_getstampedbuffer(unsigned long code, 
+				    int size)
+{
+	unsigned long *prev_slot, *next_slot;
+ retry:
+	do {
+		prev_slot=(unsigned long *)fut_next_slot;
+		next_slot=(unsigned long*)((unsigned long)prev_slot+size);
+	} while (prev_slot != ma_cmpxchg(&fut_next_slot, 
+					 prev_slot, next_slot));
+
+
+	if (tbx_unlikely(next_slot > fut_last_slot)) {
+	  if(allow_fut_flush) {
+	    /* flush the buffer to disk */
+
+	    if (prev_slot > fut_last_slot)
+	      /* another thread is going to flush the buffer */
+	      goto retry;
+
+	    pthread_mutex_lock(&fut_flush_lock);
+
+	    if ((prev_slot + size > fut_last_slot)
+		&& (prev_slot <= fut_last_slot)) {
+
+	      fut_flush(NULL, prev_slot, 1);
+	      __fut_reset_pointers();
+	    }
+
+	    pthread_mutex_unlock(&fut_flush_lock);
+
+	    goto retry;
+	  } else {
+	    /* stop recording events */
+	    fut_active=0;
+	    /* Pourquoi on restaure ici ? Pas de race condition possible ? */
+	    fut_next_slot = prev_slot;
+	    return trash_buffer;
+	  }
+	}
+
+	fxt_trace_user_raw_t *rec=(fxt_trace_user_raw_t *)prev_slot;
+
+	return __fut_record_event(rec, code);
+}
+
 #endif /* PROFILE_NEW_FORMAT */
