@@ -29,6 +29,7 @@
 #include <sys/utsname.h>
 #include <sys/types.h>
 #include <sys/times.h>
+#include <pthread.h>
 #include "fxt.h"
 #include "fut.h"
 #include "fxt-tools.h"
@@ -67,6 +68,39 @@ static char	*bufptr = NULL;
 /*	number of contiguous allocated bytes pointed to by bufptr */
 static unsigned int	nallocated = 0;
 
+/* filename of the output trace */
+static char *fut_filename = NULL;
+/* file descriptor of the output trace */
+static int fut_fd = 0;
+
+/* if !=0, activate the recording of thread id */
+int record_tid_activated = 0;
+
+/* if !=0, activate automatic buffer dumping: when the buffer is full, it is dumped to disk.
+ * This permits to record trace larger than the buffer, but this may kill the performance
+ */
+int allow_fut_flush = 0;
+
+pthread_mutex_t fut_flush_lock;
+
+static int already_flushed = 0;
+
+void fut_set_filename(char* filename)
+{
+  if(fut_filename) {
+    if(already_flushed)
+      fprintf(stderr, "Warning: change output file name to %s, but some events have been saved in file %s\n",
+	      filename, fut_filename);
+    free(fut_filename);
+  }
+  asprintf(&fut_filename, "%s", filename);
+}
+
+void __fut_reset_pointers()
+{
+  fut_next_slot = (unsigned long *)bufptr;
+}
+
 void dumptime( time_t *the_time, clock_t *the_jiffies)
 {
   struct tms cur_time;
@@ -79,8 +113,6 @@ void dumptime( time_t *the_time, clock_t *the_jiffies)
 }
 
 
-int record_tid_activated = 0;
-
 /* activate the recording of thread id */
 void fut_enable_tid_logging()
 {
@@ -93,6 +125,21 @@ void fut_disable_tid_logging()
   record_tid_activated = 0;
 }
 
+/* activate automatic buffer dumping: when the buffer is full, it is dumped to disk.
+ * This permits to record trace larger than the buffer, but this may kill the performance
+ */
+void enable_fut_flush()
+{
+  allow_fut_flush = 1;
+}
+
+/* disactivate automatic buffer dumping
+ */
+void disable_fut_flush()
+{
+  allow_fut_flush = 0;
+}
+
 /*	called once to set up tracing.
 	includes mallocing the buffer to hold the trace.
 	returns number of bytes allocated if all ok, else a negative error code.
@@ -132,7 +179,7 @@ int fut_setup( unsigned int nlongs, unsigned int keymask, unsigned int threadid
 	 * size(long), so that there could be some memory that is never
 	 * actually initialized. This also avoids page faults while tracing. */
 	memset(bufptr, 0, nbytes);
-		
+
 	fut_last_slot = (unsigned long *)(bufptr + nbytes);
 
 	/*	touch all pages so they get loaded */
@@ -161,6 +208,10 @@ int fut_setup( unsigned int nlongs, unsigned int keymask, unsigned int threadid
 	FUT_PROBE2(-1, FUT_CALIBRATE2_CODE, 1, 2);
 	FUT_PROBE2(-1, FUT_CALIBRATE2_CODE, 1, 2);
 
+	if(allow_fut_flush) {
+	  pthread_mutex_init(&fut_flush_lock, NULL);
+	}
+
 	return nlongs;
 	}
 
@@ -205,7 +256,6 @@ int fut_keychange( int how, unsigned int keymask, unsigned int threadid )
 	return old_active;
 	}
 
-
 /*	called once when completely done with buffer.
 	returns previous value of active if all ok, else a negative error code.
 */
@@ -213,7 +263,6 @@ int fut_done( void )
 	{
 	unsigned int	old_active = fut_active;
 
-
 	fut_active = 0;
 	if( bufptr != NULL )
 		{/* previous allocation region was not released, do it now */
@@ -266,7 +315,7 @@ int fut_reset( unsigned int keymask, unsigned int threadid )
 /*	called repeatedly to copy current buffer into user space.
 	returns nlongs >= 0 if all ok, else a negative error code.
 */
-int fut_getbuffer( int *nlongs, unsigned long **buffer )
+int fut_getbuffer( int *nlongs, unsigned long **buffer, unsigned long *next_slot )
 	{
 	int				local_nlongs = 0;
 
@@ -281,7 +330,7 @@ int fut_getbuffer( int *nlongs, unsigned long **buffer )
 		}
 
 	/*	get number of ints worth of trace data currently in the buffer */
-	local_nlongs = ((char *)fut_next_slot - bufptr) / sizeof(long);
+	local_nlongs = ((char *)next_slot - bufptr) / sizeof(long);
 
 	if( nlongs != NULL )
 		*nlongs = local_nlongs;
@@ -293,43 +342,119 @@ out:
 	return local_nlongs;
 	}
 
+int fut_flush( char* filename, void* next_slot, int record_flush_events )
+{
+
+  int n, nlongs=0;
+  long size;
+  unsigned long *copy=NULL;
+  static off_t begin = 0;
+
+  dumptime(&fut_infos->stop_time,&fut_infos->stop_jiffies);
+
+  if( (n = fut_getbuffer(&nlongs, &copy, next_slot)) < 0 )
+    return n;
+
+  size = nlongs * sizeof(long);
+
+  if(!fut_fd) {
+
+    /* First time fut_flush is called. Open the output file and write headers */
+    if( fut_filename == NULL )
+      fut_filename = filename ;
+
+    if( fut_filename == NULL )
+      fut_filename = DEFAULT_TRACE_FILE;
+
+    /* we can't open the file using the O_NONBLOCK option since after a call to write
+     * we are going to modify the buffer. If O_NONBLOCK is set, modifying the buffer while
+     * the OS is writting it may corrupt the output trace.
+     */
+
+    /* todo: implement a pipeline
+     * see Alexandre DENIS. "A High Performance Superpipeline Protocol for InfiniBand" in
+     * Proceedings of the 17th International Euro-Par Conference for instance
+     */
+    if( (fut_fd = open(fut_filename, O_WRONLY|O_CREAT|O_TRUNC, 0666)) < 0 )
+      return fut_fd;
+
+    already_flushed = 1;
+    /* write header */
+    fxt_fdwrite(fut, fut_fd);
+
+    /* begin a block of events */
+    fxt_fdevents_start(fut, fut_fd, FXT_TRACE_USER_RAW);
+
+    if( (begin=lseek(fut_fd,0,SEEK_CUR)) < 0 ) {
+      perror("getting block end seek");
+      exit(EXIT_FAILURE);
+    }
+
+  }
+
+  fxt_trace_user_raw_t start_flush_event;
+  if(record_flush_events)
+    __fut_record_event(&start_flush_event, ((FUT_START_FLUSH_CODE)<<8) | 0x01);
+
+  /* dump the buffer to disk */
+  if( write(fut_fd, (void *)copy, size) < 0 ) {
+    perror("write buffer");
+  }
+
+  fxt_trace_user_raw_t stop_flush_event;
+  if(record_flush_events) {
+    __fut_record_event(&stop_flush_event, ((FUT_STOP_FLUSH_CODE)<<8) | 0x01);
+
+    if( write(fut_fd, (void *)&start_flush_event, FUT_SIZE(0)) < 0 ) {
+	perror("write buffer");
+    }
+    if( write(fut_fd, (void *)&stop_flush_event, FUT_SIZE(0)) < 0 ) {
+	perror("write buffer");
+    }
+  }
+
+  fut_infos->page_size = size;
+
+  off_t end;
+  if( (end=lseek(fut_fd,0,SEEK_CUR)) < 0 ) {
+    perror("getting block end seek");
+    exit(EXIT_FAILURE);
+  }
+
+  /* Let's call fxt_fdevents_stop so that the data on disk is already correct.
+   * if the program crashes and can't manage to call fut_endup, the trace
+   * is still readable.
+   */
+  fxt_fdevents_stop(fut, fut_fd);
+
+  if( (lseek(fut_fd, end,SEEK_SET)) < 0 ) {
+    perror("getting block end seek");
+    exit(EXIT_FAILURE);
+  }
+
+  return nlongs;
+}
+
 int fut_endup( char *filename )
 	{
-	int n, nlongs=0, fd;
-	long size;
-	unsigned long *copy=NULL;
 
-	/* stop all futher tracing */
-	fut_active = 0;
+	  /* stop all futher tracing */
+	  fut_active = 0;
 
-	dumptime(&fut_infos->stop_time,&fut_infos->stop_jiffies);
+	  /* dump the buffer to disk */
+	  int nlongs = fut_flush(filename, fut_next_slot, 0);
+	  if(nlongs < 0)
+	    return nlongs;
 
-	if( (n = fut_getbuffer(&nlongs, &copy)) < 0 )
-		return n;
+	  /* end a block of events */
+	  fxt_fdevents_stop(fut, fut_fd);
 
-	size = nlongs * sizeof(long);
+	  if( close(fut_fd) < 0 )
+	    perror(fut_filename);
 
-	if( filename == NULL )
-		filename = DEFAULT_TRACE_FILE;
+	  __fut_reset_pointers();
+	  //fut_done(); // Removed by Raymond
+	  fut_active = 1;
 
-	if( (fd = open(filename, O_WRONLY|O_CREAT|O_TRUNC, 0666)) < 0 )
-		return fd;
-
-	fut_infos->page_size = size;
-	fxt_fdwrite(fut,fd);
-
-	fxt_fdevents_start(fut, fd, FXT_TRACE_USER_RAW);
-	//if( write(fd, (void *)&size, sizeof(size)) < sizeof(size) )
-	//  perror("write buffer's size");
-	
-	if( write(fd, (void *)copy, size) < 0 )
-		perror("write buffer");
-	fxt_fdevents_stop(fut, fd);
-
-	if( close(fd) < 0 )
-		perror(filename);
-
-	//fut_done(); // Removed by Raymond
-
-	return nlongs;
+	  return nlongs;
 	}
