@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004 Samuel Thibault <samuel.thibault@ens-lyon.org>
+ *  Copyright (C) 2004, 2011 Samuel Thibault <samuel.thibault@ens-lyon.org>
  *
  * This program is free software ; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -142,7 +142,7 @@ extern const char *fxt_find_name(fxt_t fxt, unsigned long code, int keep_entry,
 
 extern void fxt_get_symbols(fxt_t fxt, const char *file, fxt_symbol_source source, unsigned long base);
 extern void fut_get_mysymbols(fxt_t fxt);
-extern int fxt_load_symbols(fxt_t fxt, int fd);
+extern int fxt_load_symbols(fxt_t fxt, FILE *fstream);
 extern const char *fxt_lookup_symbol(fxt_t fxt, unsigned long address);
 
 /******************************************************************************
