@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003, 2004  Samuel Thibault <samuel.thibault@ens-lyon.org>
+ *  Copyright (C) 2003, 2004, 2011  Samuel Thibault <samuel.thibault@ens-lyon.org>
  *
  * This program is free software ; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -398,7 +398,7 @@ int fxt_record_symbols( fxt_t fxt, int fd )
 	return 0;
 	}
 
-int fxt_load_symbols( fxt_t fxt, int fd )
+int fxt_load_symbols( fxt_t fxt, FILE *fstream )
 	{
 	//static int i;
 	struct address_entry	*entry;
@@ -423,7 +423,7 @@ restart:
 		perror("allocating System map entry name");
 		return -1;
 		}
-	if( entry->len && read(fd, name, entry->len) < entry->len )
+	if( entry->len && fread(name, 1, entry->len, fstream) < entry->len )
 		{
 		perror("reading System map name");
 		return -1;
